@@ -1,5 +1,8 @@
 package Dancer2::Plugin::FormValidator::Role::ProfileHasMessages;
 
+use strict;
+use warnings;
+
 use Moo::Role;
 
 with 'Dancer2::Plugin::FormValidator::Role::Profile',
